﻿namespace Eq3.misc.usb_bat
{
    using System;
    using System.Text;
    using System.Text.RegularExpressions;
    using System.Threading;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Media;
    using System.Windows.Threading;
    using Microsoft.Win32;


    /// <summary>
    /// Interaktionslogik für Window1.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        // Using a DependencyProperty as the backing store for Message.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty MessageProperty =
            DependencyProperty.Register("Message", typeof(string), typeof(MainWindow), new UIPropertyMetadata(string.Empty));
        
        // Using a DependencyProperty as the backing store for Time.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TimeProperty =
            DependencyProperty.Register("Time", typeof(double), typeof(MainWindow), new UIPropertyMetadata(0.0));
        
        // Using a DependencyProperty as the backing store for TimeString.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TimeStringProperty =
            DependencyProperty.Register("TimeString", typeof(string), typeof(MainWindow), new UIPropertyMetadata("unendlich"));
        
       // Using a DependencyProperty as the backing store for currentStatus.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty currentStatusProperty =
            DependencyProperty.Register("currentStatus", typeof(BATStatus), typeof(MainWindow), new UIPropertyMetadata(null));

        private UsbBatDevice usbBatDevice;
        private int bufferSize = 64;

        public MainWindow()
        {
            InitializeComponent();
            this.DataContext = this;
            this.currentStatus = new BATStatus();

            // Initialisiert das UsbBatDevice
            this.usbBatDevice = new UsbBatDevice();
            this.usbBatDevice.ReportReceived += new UsbLibrary.DataRecievedEventHandler(this.UsbBat_ReportReceived);
            this.usbBatDevice.DeviceDisconnected += new EventHandler(this.UsbBat_DeviceDisconnected);
        }

        private delegate void ReportReceivedDelegate(byte[] data);

        /// <summary>
        /// Nachricht bzw Statusmeldung
        /// </summary>
        public string Message
        {
            get { return (string)GetValue(MessageProperty); }
            set { SetValue(MessageProperty, value); }
        }

        /// <summary>
        /// Zeitauswahl des Schiebereglers in Sekunden
        /// </summary>
        public double Time
        {
            get { return (double)GetValue(TimeProperty); }
            set { SetValue(TimeProperty, value); }
        }

        /// <summary>
        /// Zeitinformation als Textdarstellen
        /// </summary>
        public string TimeString
        {
            get { return (string)GetValue(TimeStringProperty); }
            set { SetValue(TimeStringProperty, value); }
        }
        
        public BATStatus currentStatus
        {
            get { return (BATStatus)GetValue(currentStatusProperty); }
            set { SetValue(currentStatusProperty, value); }
        }
                
        internal static void DisplayException(string moduleName, Exception e)
        {
            string message = string.Empty;
            string caption = string.Empty;

            message = "Exception: " + e.Message + "\r\nModule: " + moduleName + "\r\nMethod: " + e.TargetSite.Name;

            caption = "Unexpected Exception";

            MessageBox.Show(message, caption, MessageBoxButton.OK);
        }

        /// <summary>
        /// Ereignis, wenn Gerät abgezogen wird
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void UsbBat_DeviceDisconnected(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// Daten vom UsbBat empfangen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        private void UsbBat_ReportReceived(object sender, UsbLibrary.DataRecievedEventArgs args)
        {
            Dispatcher.BeginInvoke(new ReportReceivedDelegate(this.SetReport), args.data);
        }

        /// <summary>
        /// Wertet das Ergebnis des Befehls aus.
        /// </summary>
        /// <param name="currentRecord">Bytefolge des Ergebnisses.</param>
        private void SetReport(byte[] currentRecord)
        {
            if (currentRecord != null)
            {
                if (currentRecord[2] == 0xA0)
                {
                    // Normalerantwortrahmen
                    switch (currentRecord[3])
                    {
                        case 0x00:
                            this.Message = "PC-Anweisung erfolgreich ausgeführt";
                            break;
                        case 0x01:
                            this.Message = "Firmware-Version: " + (currentRecord[4] / 16) + "."  + currentRecord[4] % 16;
                            break;
                        case 0x02:
                            this.Message = "Unbekannte Befehls-ID";
                            break;
                        case 0x03:
                            this.Message = "Falsch angegebene Byte-Anzahl zur Befehls-ID";
                            break;

                        default:
                            break;
                    }
                }
                else if (currentRecord[2] == 0xF5)
                {
                    // Statusinformation der Taster und des Inkrementalgebers
                    // this.Message = currentRecord[3].ToString("X2") + currentRecord[4].ToString("X2") + currentRecord[5].ToString("X2") + currentRecord[6].ToString("X2");
                    byte[] temp = new byte[4];
                    temp[0] = currentRecord[3];
                    temp[1] = currentRecord[4];
                    temp[2] = currentRecord[5];
                    temp[3] = currentRecord[6];

                    this.currentStatus = BATStatus.FromBytes(temp);
                }
            }
            else
            {
                this.Message = "Fehler bei der Kommunikation, keine Daten empfangen";
            }
        }

        /// <summary>
        /// bei Start der Anwendung nach dem Geraet suchen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Initialized(object sender, EventArgs e)
        {
            this.bufferSize = 64;
        }

        /// <summary>
        /// Button Firmware Version
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonFirmware_Click(object sender, RoutedEventArgs e)
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x01;
            outputReportBuffer[2] = 0xF0;

            for (int i = 3; i < this.bufferSize; i++)
            {
                outputReportBuffer[i] = 0x00;
            }

            this.usbBatDevice.WriteData(outputReportBuffer);
        }

        /// <summary>
        /// Button Displayhinterleuchtung ein
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDispayOn_Click(object sender, RoutedEventArgs e)
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x02;
            outputReportBuffer[2] = 0xF1;
            outputReportBuffer[3] = this.TimeByte();

            for (int i = 4; i < this.bufferSize; i++)
            {
                outputReportBuffer[i] = 0x00;
            }

            this.usbBatDevice.WriteData(outputReportBuffer);
        }

        /// <summary>
        /// Button Displayhinterleuchtung aus
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDispayOff_Click(object sender, RoutedEventArgs e)
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x02;
            outputReportBuffer[2] = 0xF2;

            if ((bool)this.DisplayMode.IsChecked)
            {
                outputReportBuffer[3] = 0x01;
            }
            else
            {
                outputReportBuffer[3] = 0x00;
            }

            for (int i = 4; i < this.bufferSize; i++)
            {
                outputReportBuffer[i] = 0x00;
            }

            this.usbBatDevice.WriteData(outputReportBuffer);
        }

        /// <summary>
        /// Button Pieper ein
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonPiepOn_Click(object sender, RoutedEventArgs e)
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x02;
            outputReportBuffer[2] = 0xF3;
            outputReportBuffer[3] = this.TimeByte();

            for (int i = 4; i < this.bufferSize; i++)
            {
                outputReportBuffer[i] = 0x00;
            }

            this.usbBatDevice.WriteData(outputReportBuffer);
        }

        /// <summary>
        /// Button Pieper aus
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonPiepOff_Click(object sender, RoutedEventArgs e)
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x01;
            outputReportBuffer[2] = 0xF4;

            for (int i = 3; i < this.bufferSize; i++)
            {
                outputReportBuffer[i] = 0x00;
            }

            this.usbBatDevice.WriteData(outputReportBuffer);
        }

        /// <summary>
        /// Button Status der Taster abfragen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonStatus_Click(object sender, RoutedEventArgs e)
        {
            this.GetStatus();
        }
        
        /// <summary>
        /// Button zuruecksetzen des Inkrementalgebers
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonInkReset_Click(object sender, RoutedEventArgs e)
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x01;
            outputReportBuffer[2] = 0xF6;

            for (int i = 3; i < this.bufferSize; i++)
            {
                outputReportBuffer[i] = 0x00;
            }

            this.usbBatDevice.WriteData(outputReportBuffer);
        }

        /// <summary>
        /// Setzen des Flags das automatisch bei Benutzereingaben diese gesendet werden sollen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ComboBox tempComboBox = (ComboBox)sender;
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x02;
            outputReportBuffer[2] = 0xF7;
            outputReportBuffer[3] = (byte)tempComboBox.SelectedIndex;

            for (int i = 4; i < this.bufferSize; i++)
            {
                outputReportBuffer[i] = 0x00;
            }

            if (this.usbBatDevice != null)
            {
                this.usbBatDevice.WriteData(outputReportBuffer);
            }
        }

        /// <summary>
        /// Slider für die Einschaltdauer wurde veraendert
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Slider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (this.Time < 0.01)
            {
                this.TimeString = "unendlich"; // Zeit von 0 entspricht unendlich
            }
            else
            {
                this.TimeString = this.Time.ToString("F") + " s";
            }
        }

        /// <summary>
        /// Rechnet die Zeit von double um in ein zuübertragendes Byte
        /// </summary>
        /// <returns>Zeit als Byte</returns>
        private byte TimeByte()
        {
            byte result = 0x00; // default unendlich

            if (this.Time >= 0.01)
            {
                result = (byte)((this.Time + 0.005) * 100);
            }

            return result;
        }

        /// <summary>
        /// Fragt den Status von Geraet ab
        /// </summary>
        private void GetStatus()
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x01;
            outputReportBuffer[2] = 0xF5;

            for (int i = 3; i < this.bufferSize; i++)
            {
                outputReportBuffer[i] = 0x00;
            }

            this.usbBatDevice.WriteData(outputReportBuffer);
        }

        /// <summary>
        /// Button zum Schreiben einer Page
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonWritePage_Click(object sender, RoutedEventArgs e)
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x3E;
            outputReportBuffer[2] = (byte)(0xD0 + this.PageID.SelectedIndex);

            // HexString umwandeln in Bytes
            string temp = this.PageData.Text;

            Regex regularExpression = new Regex("^([0-9a-fA-F]*)$");

            if (regularExpression.IsMatch(temp) && (temp.Length % 2) == 0)
            {
                for (int i = 0; i < temp.Length; i += 2)
                {
                    string sub = temp.Substring(i, 2);
                    outputReportBuffer[3 + i / 2] = byte.Parse(sub, System.Globalization.NumberStyles.HexNumber);
                }

                this.usbBatDevice.WriteData(outputReportBuffer);

                this.PageData.Background = Brushes.White;
            }
            else
            {
                this.PageData.Background = Brushes.Red;
            }
        }

        /// <summary>
        /// Button zum Setzen eines Textes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonWriteString_Click(object sender, RoutedEventArgs e)
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x18;
            outputReportBuffer[2] = 0xD8;
            outputReportBuffer[3] = (byte)(this.ZeileID.SelectedIndex * 16 + this.PositionsID.SelectedIndex);

            if (this.Position.Text.Length > 0)
            {
                outputReportBuffer[4] = byte.Parse(this.Position.Text);
            }

            // Text erweitern auf die maximalen 20 Zeichen mit Leerzeichen um alte vorhandene Zeichen zu überschreiben
            string text = this.String.Text;
            for (int i = text.Length; i <= 20; i++)
            {
                text += " ";
            }

            // Ascii Zeichen in Bytes umwandeln
            Encoding enc = new ASCIIEncoding();
            enc.GetBytes(text).CopyTo(outputReportBuffer, 5);

            this.usbBatDevice.WriteData(outputReportBuffer);
        }

        /// <summary>
        /// Button zum Displayinhalt loeschen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonClearDisplay_Click(object sender, RoutedEventArgs e)
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x01;
            outputReportBuffer[2] = 0xD9;

            for (int i = 3; i < this.bufferSize; i++)
            {
                outputReportBuffer[i] = 0x00;
            }

            this.usbBatDevice.WriteData(outputReportBuffer);
        }

        /// <summary>
        /// Button zum Page Inhalt loeschen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonClearPage_Click(object sender, RoutedEventArgs e)
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x02;
            outputReportBuffer[2] = 0xDA;
            outputReportBuffer[3] = (byte)(this.PagedID.SelectedIndex);

            for (int i = 4; i < this.bufferSize; i++)
            {
                outputReportBuffer[i] = 0x00;
            }

            this.usbBatDevice.WriteData(outputReportBuffer);
        }

        /// <summary>
        /// Button zum Zeile loeschen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonClearLine_Click(object sender, RoutedEventArgs e)
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x02;
            outputReportBuffer[2] = 0xDB;
            outputReportBuffer[3] = (byte)(this.ZeiledID.SelectedIndex);

            for (int i = 4; i < this.bufferSize; i++)
            {
                outputReportBuffer[i] = 0x00;
            }

            this.usbBatDevice.WriteData(outputReportBuffer);
        }

        /// <summary>
        /// Button um ein Bild zu laden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonWriteImage_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog opendialog = new OpenFileDialog();
            opendialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            opendialog.Filter = "Bitmap|*.bmp";

            if (opendialog.ShowDialog() == true)
            {
                // Maximale Größe 122 * 32 Pixel
                System.Drawing.Image import = System.Drawing.Image.FromFile(opendialog.FileName);

                System.Drawing.Bitmap tempBitmap = new System.Drawing.Bitmap(import);

                byte[] page0 = new byte[61];
                byte[] page1 = new byte[61];
                byte[] page2 = new byte[61];
                byte[] page3 = new byte[61];
                byte[] page4 = new byte[61];
                byte[] page5 = new byte[61];
                byte[] page6 = new byte[61];
                byte[] page7 = new byte[61];


                for (int x = 0; x < tempBitmap.Width; x++)
                {
                    for (int y = 0; y < tempBitmap.Height; y++)
                    {
                        System.Drawing.Color temp = tempBitmap.GetPixel(x, y);

                        // Nur Schwarze Pixel werden gesetzt
                        if (temp.B <= 50 && temp.G <= 50 && temp.R <= 50)
                        {
                            // Aufteilen des Bildes auf die Pages 0-7
                            // (y  0- 7) Page0 (0-60)  Page1 (61 - 121)
                            // (y  8-15) Page2 (0-60)  Page3 (61 - 121)
                            // (y 16-23) Page4 (0-60)  Page5 (61 - 121)
                            // (y 24-31) Page6 (0-60)  Page7 (61 - 121)
                            if (y < 8 && x < 61)
                            {
                                page0[x % 61] += (byte)(0x01 << y % 8);
                            }
                            else if (y < 8 && x < 122)
                            {
                                page1[x % 61] += (byte)(0x01 << y % 8);
                            }
                            else if (y < 16 && x < 61)
                            {
                                page2[x % 61] += (byte)(0x01 << y % 8);
                            }
                            else if (y < 16 && x < 122)
                            {
                                page3[x % 61] += (byte)(0x01 << y % 8);
                            }
                            else if (y < 24 && x < 61)
                            {
                                page4[x % 61] += (byte)(0x01 << y % 8);
                            }
                            else if (y < 24 && x < 122)
                            {
                                page5[x % 61] += (byte)(0x01 << y % 8);
                            }
                            else if (y < 32 && x < 61)
                            {
                                page6[x % 61] += (byte)(0x01 << y % 8);
                            }
                            else if (y < 32 && x < 122)
                            {
                                page7[x % 61] += (byte)(0x01 << y % 8);
                            }
                        }
                    }
                }

                // Setzen von Page0
                byte[] outputReportBuffer = new byte[this.bufferSize];
                outputReportBuffer[0] = 0x01;
                outputReportBuffer[1] = 0x3E;
                outputReportBuffer[2] = 0xD0;
                page0.CopyTo(outputReportBuffer, 3);

                this.usbBatDevice.WriteData(outputReportBuffer);

                Thread.Sleep(10);

                // Setzen von Page1
                outputReportBuffer[0] = 0x01;
                outputReportBuffer[1] = 0x3E;
                outputReportBuffer[2] = 0xD1;
                page1.CopyTo(outputReportBuffer, 3);

                this.usbBatDevice.WriteData(outputReportBuffer);

                Thread.Sleep(10);

                // Setzen von Page2
                outputReportBuffer[0] = 0x01;
                outputReportBuffer[1] = 0x3E;
                outputReportBuffer[2] = 0xD2;
                page2.CopyTo(outputReportBuffer, 3);

                this.usbBatDevice.WriteData(outputReportBuffer);

                Thread.Sleep(10);

                // Setzen von Page3
                outputReportBuffer[0] = 0x01;
                outputReportBuffer[1] = 0x3E;
                outputReportBuffer[2] = 0xD3;
                page3.CopyTo(outputReportBuffer, 3);

                this.usbBatDevice.WriteData(outputReportBuffer);

                Thread.Sleep(10);

                // Setzen von Page4
                outputReportBuffer[0] = 0x01;
                outputReportBuffer[1] = 0x3E;
                outputReportBuffer[2] = 0xD4;
                page4.CopyTo(outputReportBuffer, 3);

                this.usbBatDevice.WriteData(outputReportBuffer);

                Thread.Sleep(10);

                // Setzen von Page5
                outputReportBuffer[0] = 0x01;
                outputReportBuffer[1] = 0x3E;
                outputReportBuffer[2] = 0xD5;
                page5.CopyTo(outputReportBuffer, 3);

                this.usbBatDevice.WriteData(outputReportBuffer);

                Thread.Sleep(10);

                // Setzen von Page6
                outputReportBuffer[0] = 0x01;
                outputReportBuffer[1] = 0x3E;
                outputReportBuffer[2] = 0xD6;
                page6.CopyTo(outputReportBuffer, 3);

                this.usbBatDevice.WriteData(outputReportBuffer);

                Thread.Sleep(10);

                // Setzen von Page7
                outputReportBuffer[0] = 0x01;
                outputReportBuffer[1] = 0x3E;
                outputReportBuffer[2] = 0xD7;
                page7.CopyTo(outputReportBuffer, 3);

                this.usbBatDevice.WriteData(outputReportBuffer);
            }
        }

        private void ButtonDisplayInit_Click(object sender, RoutedEventArgs e)
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x01;
            outputReportBuffer[2] = 0xDC;

            for (int i = 3; i < this.bufferSize; i++)
            {
                outputReportBuffer[i] = 0x00;
            }

            this.usbBatDevice.WriteData(outputReportBuffer);
        }

        private void ButtonDeviceReset_Click(object sender, RoutedEventArgs e)
        {
            byte[] outputReportBuffer = new byte[this.bufferSize];
            outputReportBuffer[0] = 0x01;
            outputReportBuffer[1] = 0x01;
            outputReportBuffer[2] = 0xF8;

            for (int i = 3; i < this.bufferSize; i++)
            {
                outputReportBuffer[i] = 0x00;
            }

            this.usbBatDevice.WriteData(outputReportBuffer);
        }
    }
}
